<?php
/*
 * List Model
 *
 * @package Joomla.Administrator
 * @subpackage Fabrik
 * @since		1.6
 * @copyright Copyright (C) 2005 Rob Clayburn. All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 */

// No direct access.
defined('_JEXEC') or die;

require_once('fabmodeladmin.php');

class FabrikModelList extends FabModelAdmin
{
	/**
	 * @var		string	The prefix to use with controller messages.
	 * @since	1.6
	 */
	protected $text_prefix = 'COM_FABRIK_LIST';

	protected $formModel = null;

	/** @var object model - front end table model */
	protected $feListModel = null;

	/** @var object currently loaded list row */
	protected $tables = array();

	protected $pluginType = 'List';

	/**
	 * Returns a reference to the a Table object, always creating it.
	 *
	 * @param	type	The table type to instantiate
	 * @param	string	A prefix for the table class name. Optional.
	 * @param	array	Configuration array for model. Optional.
	 * @return	JTable	A database object
	 * @since	1.6
	 */

	public function getTable($type = 'List', $prefix = 'FabrikTable', $config = array())
	{
		$sig = $type.$prefix.implode('.', $config);
		if (!array_key_exists($sig, $this->tables)) {
			$config['dbo'] = FabriKWorker::getDbo();
			$this->tables[$sig] = FabTable::getInstance($type, $prefix, $config);
		}
		return $this->tables[$sig];
	}

	/**
	 * Method to get the record form.
	 *
	 * @param	array	$data		Data for the form.
	 * @param	boolean	$loadData	True if the form is to load its own data (default case), false if not.
	 * @return	mixed	A JForm object on success, false on failure
	 * @since	1.6
	 */
	public function getForm($data = array(), $loadData = true)
	{
		// Get the form.
		$form = $this->loadForm('com_fabrik.list', 'list', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form)) {
			return false;
		}
		$form->model = $this;
		return $form;
	}

	/**
	 * Method to get the confirm list delete form.
	 *
	 * @param	array	$data		Data for the form.
	 * @param	boolean	$loadData	True if the form is to load its own data (default case), false if not.
	 * @return	mixed	A JForm object on success, false on failure
	 * @since	1.6
	 */
	public function getConfirmDeleteForm($data = array(), $loadData = true)
	{
		// Get the form.
		$form = $this->loadForm('com_fabrik.confirmdelete', 'confirmlistdelete', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form)) {
			return false;
		}
		return $form;
	}

	/**
	 * Method to get the data that should be injected in the form.
	 *
	 * @return	mixed	The data for the form.
	 * @since	1.6
	 */

	protected function loadFormData()
	{
		// Check the session for previously entered form data.
		$data = JFactory::getApplication()->getUserState('com_fabrik.edit.list.data', array());

		if (empty($data)) {
			$data = $this->getItem();
		}

		return $data;
	}

	/**
	 * Method to change the published state of one or more records.
	 *
	 * @param	array	A list of the primary keys to change.
	 * @param	int		The value of the published state.
	 * @return	boolean	True on success.
	 * @since	1.6
	 */
	function publish(&$pks, $value = 1)
	{
		// Initialise variables.
		$dispatcher	= JDispatcher::getInstance();
		$user		= JFactory::getUser();
		$table		= $this->getTable();
		$pks		= (array) $pks;

		// Include the content plugins for the change of state event.
		JPluginHelper::importPlugin('content');

		// Access checks.
		foreach ($pks as $i => $pk) {
			if ($table->load($pk)) {
				if (!$this->canEditState($table)) {
					// Prune items that you can't change.
					unset($pks[$i]);
					JError::raiseWarning(403, JText::_('JLIB_APPLICATION_ERROR_EDIT_STATE_NOT_PERMITTED'));
				}
			}
		}

		// Attempt to change the state of the records.
		if (!$table->publish($pks, $value, $user->get('id'))) {
			$this->setError($table->getError());
			return false;
		}

		$context = $this->option.'.'.$this->name;

		// Trigger the onContentChangeState event.
		$result = $dispatcher->trigger($this->event_change_state, array($context, $pks, $value));
		if (in_array(false, $result, true)) {
			$this->setError($table->getError());
			return false;
		}

		return true;
	}

	/**
	 *
	 * @param bol add slashes to reutrn data
	 */

	protected function getFilterJoinDd($addslashes = true, $name = 'join')
	{
		$aConditions = array();
		$aConditions[] = JHTML::_('select.option', 'AND');
		$aConditions[] = JHTML::_('select.option', 'OR');
		$dd = str_replace("\n", "", JHTML::_('select.genericlist', $aConditions, $name, "class=\"inputbox\"  size=\"1\" ", 'value', 'text', ''));
		if ($addslashes) {
			$dd = addslashes($dd);
		}
		return $dd;
	}

	/**
	 *@param bol add slashes to reutrn data
	 *@param string name of the drop down
	 *@param int mode - states what values get put into drop down
	 */

	protected function getFilterConditionDd($addslashes = true, $name = 'conditions', $mode = 1)
	{
		$aConditions = array();
		switch ($mode) {
			case 1:
				/* used for search filter */
				$aConditions[] = JHTML::_('select.option', '<>', 'NOT EQUALS');
				$aConditions[] = JHTML::_('select.option', '=', 'EQUALS');
				$aConditions[] = JHTML::_('select.option', 'like', 'BEGINS WITH');
				$aConditions[] = JHTML::_('select.option', 'like', 'CONTAINS');
				$aConditions[] = JHTML::_('select.option', 'like', 'ENDS WITH');
				$aConditions[] = JHTML::_('select.option', '>', 'GREATER THAN');
				$aConditions[] = JHTML::_('select.option', '>=', 'GREATER THAN OR EQUALS');
				$aConditions[] = JHTML::_('select.option', '<', 'LESS THAN');
				$aConditions[] = JHTML::_('select.option', '<=', 'LESS THAN OR EQUALS');
				break;
			case 2:
				/* used for prefilter */
				$aConditions[] = JHTML::_('select.option', 'equals', 'EQUALS');
				$aConditions[] = JHTML::_('select.option', 'notequals', 'NOT EQUAL TO');
				$aConditions[] = JHTML::_('select.option', 'begins', 'BEGINS WITH');
				$aConditions[] = JHTML::_('select.option', 'contains', 'CONTAINS');
				$aConditions[] = JHTML::_('select.option', 'ends', 'ENDS WITH');
				$aConditions[] = JHTML::_('select.option', '>', 'GREATER THAN');
				$aConditions[] = JHTML::_('select.option', '>=', 'GREATER THAN OR EQUALS');
				$aConditions[] = JHTML::_('select.option', '<', 'LESS THAN');
				$aConditions[] = JHTML::_('select.option', 'IS NULL', 'IS NULL');
				$aConditions[] = JHTML::_('select.option', '<=', 'LESS THAN OR EQUALS');
				$aConditions[] = JHTML::_('select.option', 'in', 'IN');
				$aConditions[] = JHTML::_('select.option', 'not_in', 'NOT IN');
				$aConditions[] = JHTML::_('select.option', 'earlierthisyear', JText::_('COM_FABRIK_EARLIER_THIS_YEAR'));
				$aConditions[] = JHTML::_('select.option', 'laterthisyear', JText::_('COM_FABRIK_LATER_THIS_YEAR'));
				break;
		}
		$dd = str_replace("\n", "", JHTML::_('select.genericlist',  $aConditions, $name, "class=\"inputbox\"  size=\"1\" ", 'value', 'text', ''));
		if ($addslashes) {
			$dd = addslashes( $dd);
		}
		return $dd;
	}

	protected function getCnn()
	{
		$item = $this->getItem();
		JModel::addIncludePath(JPATH_SITE.DS.'components'.DS.'com_fabrik'.DS.'models');
		$connModel 	=& JModel::getInstance('Connection', 'FabrikFEModel');
		$connModel->setId($item->connection_id);
		$connModel->getConnection($item->connection_id);
		return $connModel;
	}

	/**
	 * create the js that manages the edit list view page
	 */

	public function getJs()
	{
		$abstractPlugins = $this->getAbstractPlugins();
		$connModel = $this->getCnn();
		$plugins = $this->getPlugins();
		$item = $this->getItem();
		JModel::addIncludePath(JPATH_SITE.DS.'components'.DS.'com_fabrik'.DS.'models');
		$pluginManager = JModel::getInstance('Pluginmanager', 'FabrikFEModel');
		JText::script('COM_FABRIK_ACTION');
		JText::script('COM_FABRIK_DO');
		JText::script('COM_FABRIK_IN');
		JText::script('COM_FABRIK_ON');
		JText::script('COM_FABRIK_OPTIONS');
		JText::script('COM_FABRIK_JOIN');
		JText::script('COM_FABRIK_FIELD');
		JText::script('COM_FABRIK_CONDITION');
		JText::script('COM_FABRIK_VALUE');
		JText::script('COM_FABRIK_EVAL');
		JText::script('COM_FABRIK_APPLY_FILTER_TO');
		JText::script('COM_FABRIK_DELETE');
		JText::script('JYES');
		JText::script('JNO');
		JText::script('COM_FABRIK_QUERY');
		JTEXT::script('COM_FABRIK_NO_QUOTES');
		JText::script('COM_FABRIK_TEXT');
		JText::script('COM_FABRIK_TYPE');
		JText::script('COM_FABRIK_PLEASE_SELECT');
		JText::script('COM_FABRIK_GROUPED');
		JText::script('COM_FABRIK_TO');
		JText::script('COM_FABRIK_FROM');
		JText::script('COM_FABRIK_JOIN_TYPE');
		JText::script('COM_FABRIK_FROM_COLUMN');
		JText::script('COM_FABRIK_TO_COLUMN');

		$joinTypeOpts = array();
		$joinTypeOpts[] = array('inner', JText::_('INNER JOIN'));
		$joinTypeOpts[] = array('left', JText::_('LEFT JOIN'));
		$joinTypeOpts[] = array('right', JText::_('RIGHT JOIN'));
		$activetableOpts[] = "";
		$activetableOpts[] = $item->db_table_name;


		$joins = $this->getJoins();

		for ($i = 0; $i < count($joins); $i ++) {
			$j = $joins[$i];
			$activetableOpts[] = $j->table_join;
			$activetableOpts[] = $j->join_from_table;
		}
		$activetableOpts = array_unique($activetableOpts);
		$activetableOpts = array_values($activetableOpts);
		$opts = new stdClass();
		$opts->joinOpts = $joinTypeOpts;
		$opts->tableOpts = $connModel->getThisTables(true);
		$opts->activetableOpts = $activetableOpts;
		$opts = json_encode($opts);

		$filterOpts = new stdClass();
		$filterOpts->filterJoinDd = $this->getFilterJoinDd(false, 'jform[params][filter-join][]');
		$filterOpts->filterCondDd = $this->getFilterConditionDd(false, 'jform[params][filter-conditions][]', 2);
		$filterOpts->filterAccess 	= JHtml::_('access.level', 'jform[params][filter-access][]', $item->access);
		$filterOpts->filterAccess = str_replace(array("\n", "\r"), '', $filterOpts->filterAccess);
		$filterOpts = json_encode($filterOpts);

		$formModel =& $this->getFormModel();
		$filterfields = $formModel->getElementList('jform[params][filter-fields][]', '', false, false, true);
		$filterfields = addslashes(str_replace(array("\n", "\r"), '', $filterfields));
		$js =
	"
  window.addEvent('load', function(){

		oAdminTable = new ListForm($opts);
	oAdminTable.watchJoins();\n";

		for ($i = 0; $i < count($joins); $i ++) {
			$j = $joins[$i];
			$joinFormFields = json_encode($j->joinFormFields);
			$joinToFields =  json_encode($j->joinToFields);
			$js .= "	oAdminTable.addJoin('{$j->group_id}','{$j->id}','{$j->join_type}','{$j->table_join}',";
			$js .= "'{$j->table_key}','{$j->table_join_key}','{$j->join_from_table}', $joinFormFields, $joinToFields);\n";
		}

		$js .= "var aPlugins = [];\n";
		foreach ($abstractPlugins as $abstractPlugin) {
			$js .= "aPlugins.push(".$abstractPlugin['js'].");\n";
		}
		$js .= "controller = new ListPluginManager(aPlugins);\n";
		foreach ($plugins as $plugin) {
			$js .= "controller.addAction('".$plugin['html']."', '".$plugin['plugin']."', '".@$plugin['location']."', '".@$plugin['event']."', false);\n";
		}

		$js .= "	oAdminFilters = new adminFilters('filterContainer', '$filterfields', $filterOpts);\n";

		$form = $this->getForm();

		$afilterJoins	= $form->getValue('params.filter-join');

		$afilterFields = $form->getValue('params.filter-fields');
		$afilterConditions	= $form->getValue('params.filter-conditions');
		$afilterEval = $form->getValue('params.filter-eval');
		$afilterValues= $form->getValue('params.filter-value');
		$afilterAccess= $form->getValue('params.filter-access');
		$aGrouped = $form->getValue('params.filter-grouped');
		for ($i=0; $i < count($afilterFields); $i++) {
			$selJoin = JArrayHelper::getValue($afilterJoins, $i, 'and');
			$selFilter 	  = $afilterFields[$i];
			$grouped 	  = $aGrouped[$i];
			$selCondition = $afilterConditions[$i];
			$filerEval	  =  JArrayHelper::getValue($afilterEval, $i, '1');
			if($selCondition == '&gt;') { $selCondition = '>';}
			if($selCondition == '&lt;') { $selCondition = '<';}
			$selValue    = 	JArrayHelper::getValue($afilterValues, $i, '');
			$selAccess 	  = $afilterAccess[$i];

			//alow for multiline js variables ?
			$selValue = htmlspecialchars_decode($selValue, ENT_QUOTES);
			$selValue = json_encode($selValue);

			if ($selFilter != '') {
				$js .= "	oAdminFilters.addFilterOption('$selJoin', '$selFilter', '$selCondition', $selValue, '$selAccess', '$filerEval', '$grouped');\n";
			}
		}
		$js .= "
});";
		return $js;
	}

	/**
	 * get the table's join objects
	 * @return array
	 */

	protected function getJoins()
	{
		$item = $this->getItem();
		if ((int)$item->id === 0) {
			return array();
		}
		$db = FabrikWorker::getDbo();
		$query = $db->getQuery(true);
		$query->select('*')->from('#__{package}_joins')->where('list_id = '.(int)$item->id);
		$db->setQuery($query);
		$joins = $db->loadObjectList();
		for ($i = 0; $i < count($joins); $i++) {
			$join =& $joins[$i];
			$fields = $db->getTableFields(array($join->join_from_table, $join->table_join));
			$join->joinFormFields = array_keys($fields[$join->join_from_table]);
			$join->joinToFields = array_keys($fields[$join->table_join]);
		}
		return $joins;
	}

	/**
	 * get the possible list plug-ins that can be selected for use
	 * @return array
	 */

	public function getAbstractPlugins()
	{
		// create a new dispatcher so that we only collect admin html for validation rules
		$pluginDispatcher = new JDispatcher();

		//import the plugins and assign them to their custom dispatcher
		JPluginHelper::importPlugin('fabrik_list', null, true, $pluginDispatcher);
		$rules = array();
		//trigger the dispatcher to get the plug-in rules html

		$plugins = JPluginHelper::getPlugin('fabrik_list');

		JModel::addIncludePath(JPATH_SITE.DS.'components'.DS.'com_fabrik'.DS.'models');
		$pluginManager = JModel::getInstance('Pluginmanager', 'FabrikFEModel');

		$feListModel = JModel::getInstance('list', 'fabrikFEModel');
		$feListModel->setId($this->getState('list.id'));

		foreach ($plugins as $x => $plugin) {

			$data = array();

			$o = $pluginManager->getPlugIn($plugin->name, 'List');
			$o->getJForm()->model = $feListModel;

			// $$$ rob 0 was $x below but that rendered first set of plugins with indexes 1,2,3
			// think they should all be indexed 0
			$str = $o->onRenderAdminSettings($data, 0);
			$js = $o->onGetAdminJs($plugin->name, $plugin->name, $str);
			$str = addslashes(str_replace(array("\n", "\r"), "", $str));
			$rules[] = array('plugin'=>$plugin->name, 'html'=>$str, 'js'=>$js);
		}

		return $rules;
	}

	protected function getPluginLocation($repeatCounter)
	{
		return '';
	}

	protected function getPluginEvent($repeatCounter)
	{
		return '';
	}
	/**
	 * load up a front end form model - used in saving the table
	 * @return object front end form model
	 */

	public function getFormModel()
	{
		if (is_null($this->formModel)) {
			$config = array();
			JModel::addIncludePath(JPATH_SITE.DS.'components'.DS.'com_fabrik'.DS.'models');
			$config['dbo'] = FabrikWorker::getDbo();
			$this->formModel = JModel::getInstance('Form', 'FabrikFEModel', $config);
			$this->formModel->setDbo($config['dbo']);

			// $$$ rob commenting out as this loads up an empty form when saving a new list
			//$item = $this->getItem();
			//$this->formModel->setId($this->getState('list.form_id', $item->id));
			//$this->formModel->getForm();

			// $$$ rob isnt this wrong as normally the front end form models list model is the fe list model?
			//$this->formModel->setListModel($this);
		}
		return $this->formModel;
	}

	/**
	 * load up the front end list model so we can use some of its methods
	 * @return object front end list model
	 */

	public function getFEModel()
	{
		if (is_null($this->feListModel)) {
			JModel::addIncludePath(JPATH_SITE.DS.'components'.DS.'com_fabrik'.DS.'models');
			$this->feListModel = JModel::getInstance('List', 'FabrikFEModel');
			$this->feListModel->setState('list.id', $this->getState('list.id'));
		}
		return $this->feListModel;
	}

	public function validate($form, $data)
	{
		$params = $data['params'];
		$data = parent::validate($form, $data);
		if (!$data) {
			return false;
		}
		if (empty($data['_database_name']) && JArrayHelper::getValue($data, 'db_table_name') == '') {
			$this->setError(JText::_('COM_FABRIK_SELECT_DB_OR_ENTER_NAME'));
			return false;
		}
		//hack - must be able to add the plugin xml fields file to $form to include in validation
		// but cant see how at the moment
		$data['params'] = $params;
		return $data;
	}

	/**
	 * save the form
	 * @param array $data (the jform part of the request data)
	 */

	function save($data)
	{
		$this->populateState();
		$app 				= JFactory::getApplication();
		$db 				= FabrikWorker::getDbo();
		$user 			= JFactory::getUser();
		$config 		= JFactory::getConfig();
		$date = JFactory::getDate();
		$row 				= $this->getTable();
		$id = $data['id'];
		$row->load($id);

		$this->setState('list.id', $id);
		$this->setState('list.form_id', $row->form_id);
		$feModel = $this->getFEModel();
		$formModel = $this->getFormModel();

		if (!$row->bind($data)) {
			$this->setError($row->getError());
			return false;
		}

		$filter	= new JFilterInput(null, null, 1, 1);
		$introduction = JRequest::getVar('introduction', '', 'post', 'string', JREQUEST_ALLOWRAW);
		$row->introduction = $filter->clean($introduction);

		$aOrderBy = JRequest::getVar('order_by', array(), 'post', 'array');
		$row->order_by = json_encode($aOrderBy);

		$aOrderDir = JRequest::getVar('order_dir', array(), 'post', 'array');
		$row->order_dir = json_encode($aOrderDir);

		if (!$row->check()) {
			$this->setError($row->getError());
			return false;
		}
		$isNew = true;
		if ($id == 0) {
			if ($row->created == '') {
				$row->created = $date->toMySQL();
			}
			//save the row now
			$row->store();

			$isNew = false;
			$newtable = trim(JArrayHelper::getValue($data, '_database_name'));
			// $$$ hugh - added some more sanity checking on table name, get rid of non-alphanumeric and _
			// @TODO - should prolly use a helper for this, like FabrikString::clean()
			// but need to think about case issues first
			$newtable = preg_replace('#[^0-9a-zA-Z_]#', '_', $newtable);

			//check the entered database table doesnt already exist
			if ($newtable != '' && $this->getFEModel()->databaseTableExists($newtable)) {
				$this->setError(JText::_('COM_FABRIK_DATABASE_TABLE_ALREADY_EXISTS'));
				return false;
			}

			if (!$this->canCreateDbTable()) {
				$this->setError(Jtext::_('COM_FABRIK_INSUFFICIENT_RIGHTS_TO_CREATE_TABLE'));
				return false;
			}
			//create fabrik form
			$formModel =& $this->createLinkedForm();

			$row->form_id = $this->getState('list.form_id');
			//create fabrik group
			$groupData = array("name" => $row->label, "label" => $row->label);

			JRequest::setVar('_createGroup', 1, 'post');

			$groupId = $this->createLinkedGroup($groupData, false);

			if ($newtable == '') {
				//new fabrik table but existing db table
				$this->createLinkedElements($groupId, $data);
			} else {
				$row->db_table_name = $newtable;
				$row->db_primary_key = $newtable.'.id';
				$row->auto_inc = 1;
				$res = $feModel->createDBTable($formModel, $newtable);
			}
		}

		// 	save params - this file no longer exists? do we use models/table.xml instead??
		$params = new fabrikParams($row->params, JPATH_COMPONENT.DS.'xml'.DS.'table.xml');

		if ($row->id != 0) {
			$datenow = JFactory::getDate();
			$row->modified 		= $datenow->toMySQL();
			$row->modified_by 	= $user->get('id');
		}
		FabrikHelper::prepareSaveDate($row->publish_down);
		FabrikHelper::prepareSaveDate($row->created);
		FabrikHelper::prepareSaveDate($row->publish_up);
		$pk = JArrayHelper::getValue($data, 'db_primary_key');
		if ($pk == '') {
			$fields = $row->getFields();
			$key = $row->getKeyName();
			//store without namequotes as thats db specific
			$row->db_primary_key = $row->db_table_name.".".$key;
			$row->auto_inc = stristr($fields[$key]->Extra, 'auto_increment') ? true : false;
		}
		if (!$row->store()) {
			$this->setError($row->getError());
			return false;
		}
		$pk = $row->db_primary_key;
		$this->updateJoins($data);
		$feModel->setTable($row); //needed to ensure pk field is not quoted
		if (!$feModel->isView()) {
			// this was only run on a new table - but I've put it here so that if you upload a new table you can ensure that its columns are fixed
			//$this->makeSafeTableColumns();
			$this->updatePrimaryKey($row->db_primary_key, $row->auto_inc);
		}

		//make an array of elments and a presumed index size
		//map is then used in creating indexes
		$map = array();
		$groups =& $this->getFormModel()->getGroupsHiarachy();
		foreach ($groups as $groupModel) {
			$elementModels =& $groupModel->getMyElements();
			foreach ($elementModels as $element) {
				//int elements cant have a index size attrib
				// $$$ hugh neither can DATETIME
				$coltype = $element->getFieldDescription();
				if (JString::stristr($coltype, 'int')) {
					$size = '';
				}
				else if (JString::stristr($coltype, 'datetime')) {
					$size = '';
				}
				else {
					$size = '10';
				}
				$map[$element->getFullName(false, false, false)] = $size;
				$map[$element->getElement()->id] = $size;
			}
		}
		//update indexes (added array_key_exists check as these may be during after CSV import)
		if (!empty($aOrderBy) && array_key_exists($row->order_by, $map)) {
			foreach ($aOrderBy as $orderBy) {
				if (array_key_exists($orderBy, $map)) {
					$feModel->addIndex($orderBy, 'tableorder', 'INDEX', $map[$orderBy]);
				}
			}
		}
		if ($row->group_by !== '' && array_key_exists($row->group_by, $map)) {
			$feModel->addIndex($row->group_by, 'groupby', 'INDEX', $map["$row->group_by"]);
		}
		if ($params->get('group_by_order') !== '') {
			$feModel->addIndex($params->get('group_by_order'), 'groupbyorder', 'INDEX', $map[$params->get('group_by_order')]);
		}
		$afilterFields = $params->get('filter-fields', '', '_default', 'array');
		foreach ($afilterFields as $field) {
			if (array_key_exists($field, $map)) {
				$feModel->addIndex($field, 'prefilter', 'INDEX', $map[$field]);
			}
		}

		if (JFolder::exists(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_joomfish'.DS.'contentelements')) {
			if ($params->get('allow-data-translation')) {
				if (!$this->makeJoomfishXML()) {
					$this->setError(JTEXT::_( "Unable to make Joomfish XML file"));
					return false;
				}
			} else {
				$this->removeJoomfishXML();
			}
		}

		$pkName = $row->getKeyName();

		if (isset($row->$pkName)) {
			$this->setState($this->getName().'.id', $row->$pkName);
		}
		$this->setState($this->getName().'.new', $isNew);
		return true;

	}

	/**
	 * deals with ensuring joins are managed correctly when table is saved
	 * @param array data
	 */

	private function updateJoins(&$data)
	{
		$db = FabrikWorker::getDbo();
		$query = $db->getQuery(true);
		$query->select('*')->from('#__{package}_joins')->where('list_id = '.(int)$this->getState('list.id'));
		$db->setQuery($query);
		$aOldJoins = $db->loadObjectList();
		$params = $data['params'];
		$aOldJoinsToKeep 	= array();
		$joinModel			= JModel::getInstance('Join', 'FabrikFEModel');
		$joinIds 				= JArrayHelper::getValue($params, 'join_id', array());
		$joinTypes 			= JArrayHelper::getValue($params, 'join_type', array());
		$joinTableFrom  = JArrayHelper::getValue($params, 'join_from_table', array());
		$joinTable 			= JArrayHelper::getValue($params, 'table_join', array());
		$tableKey				= JArrayHelper::getValue($params, 'table_key', array());
		$joinTableKey		= JArrayHelper::getValue($params, 'table_join_key', array());
		$groupIds				= JArrayHelper::getValue($params, 'group_id', array());
		$jc = count($joinTypes);

		for ($i = 0; $i < $jc ; $i++) {
			$existingJoin = false;
			foreach ($aOldJoins as $oOldJoin) {
				if ($joinIds[$i] == $oOldJoin->id) {
					$existingJoin = true;
				}
			}
			//$$$rob make an index on the join element (fk)
			$els =& $this->getFEModel()->getElements();
			foreach ($els as $el) {
				if ($el->getElement()->name == $tableKey[$i]) {
					$size = JString::stristr($el->getFieldDescription(), 'int') ? '' : '10';
				}
			}
			$this->getFEModel()->addIndex($tableKey[$i], 'join', 'INDEX', $size);
			echo $existingJoin ? "existing " : " new";
			if (!$existingJoin) {
				$this->makeNewJoin($tableKey[$i], $joinTableKey[$i], $joinTypes[$i], $joinTable[$i], $joinTableFrom[$i]);
			} else {
				/* load in the exisitng join
				 * if the table_join has changed we need to create a new join
				 * (with its corresponding group and elements)
				 *  and mark the loaded one as to be deleted
				 */
				$joinModel->setId($joinIds[$i]);
				$joinModel->_join = null;
				$join =& $joinModel->getJoin();

				if ($join->table_join != $joinTable[$i]) {
					$this->makeNewJoin($tableKey[$i], $joinTableKey[$i], $joinTypes[$i], $joinTable[$i], $joinTableFrom[$i]);
				} else {
					//the talbe_join has stayed the same so we simply update the join info
					$join->table_key 		= str_replace('`', '', $tableKey[$i]);
					$join->table_join_key 	= $joinTableKey[$i];
					$join->join_type 		= $joinTypes[$i];
					$join->store();
					$aOldJoinsToKeep[] 		= $joinIds[$i];
				}
			}
		}
		/* remove non exisiting joins */
		if (is_array($aOldJoins)) {
			foreach ($aOldJoins as $oOldJoin) {
				if (!in_array($oOldJoin->id, $aOldJoinsToKeep)) {
					/* delete join */
					$join = $this->getTable('Join');
					$joinModel->setId($oOldJoin->id);
					unset($joinModel->_join);
					$joinModel->getJoin();
					$joinModel->deleteAll($oOldJoin->group_id);
				}
			}
		}
	}

	/**
	 *  new join make the group, group elements and formgroup entries for the join data
	 * @param string table key
	 * @param string join to table key
	 * @param string join type
	 * @param string join to table
	 * @param string join table
	 */

	protected function makeNewJoin($tableKey, $joinTableKey, $joinType, $joinTable, $joinTableFrom)
	{
		$db 	=& FabrikWorker::getDbo();
		$formModel 	=& $this->getFormModel();
		$aData = array(
			"name" => $this->getTable()->label ."- [" .$joinTable. "]",
			"label" =>  $joinTable,
		);
		$groupId = $this->createLinkedGroup($aData, true);
		$origTable = JRequest::getVar('db_table_name');
		JRequest::setVar('db_table_name', $joinTable);
		$this->createLinkedElements($groupId, array());
		JRequest::setVar('db_table_name', $origTable);
		$join = $this->getTable('Join');
		$join->list_id 		= $this->getState('list.id');
		$join->join_from_table = $joinTableFrom;
		$join->table_join 		= $joinTable;
		$join->table_join_key 	= $joinTableKey;
		$join->table_key 		= str_replace('`', '', $tableKey);
		$join->join_type 		= $joinType;
		$join->group_id 		= $groupId;
		if (!$join->store()) {
			return JError::raiseWarning(500, $join->getError());
		}
	}

	/**
	 * when saving a table that links to a database for the first time we
	 * need to create all the elements based on the database table fields and their
	 * column type
	 *
	 * @access private
	 * @param int group id
	 * @param array newly created table data
	 * @param array of element objects - if this is not empty then we've come from the csv import and the elements
	 * have already been defined, use this instead of the field analysis to create correctly typed elements
	 */

	private function createLinkedElements($groupId, $aTableData, $aSpecificElements = array())
	{
		$db 			= FabrikWorker::getDbo();
		$user = JFactory::getUser();
		$config		= JFactory::getConfig();
		$createdate = JFactory::getDate();
		$createdate = $createdate->toMySQL();
		$post = JRequest::get('post');
		$tableName = $post['jform']['db_table_name'];
		$formModel = $this->getFormModel();
		$pluginManager = $formModel->getPluginManager();
		$ordering = 0;
		$fabrikDb =& $this->getFEModel()->getDb();

		$groupTable = FabTable::getInstance('Group', 'FabrikTable');
		$groupTable->load($groupId);

		if (!empty($aSpecificElements)) {
			//we're asking the method to create predefined elements - e.g. when installing sample data.
			foreach ($aSpecificElements as $elementModel) {
				$element =& $elementModel->getElement();
				if ($element->label == 'id') {
					$element->hidden = 1;
					$element->primary_key = 1;
					$element->auto_increment = 1;
				} else {
					$element->hidden = 0;
				}
				$element->name = strtolower(str_replace(' ', '', $element->name));
				$element->group_id = $groupId;
				$element->created = $createdate;
				$element->created_by = $user->get('id');
				$element->created_by_alias = $user->get('username');
				$element->published = 1;
				$element->show_in_list_summary = 1;
				$element->width = 30;
				$element->height = 6;
				$element->ordering = 99999;
				$element->params = $elementModel->getDefaultAttribs();
				$element->store();
				$where = " group_id = '" . $element->group_id . "'";
				$element->reorder($where);
			}
		} else {
			//here we're importing directly from the database schema
			$query = $db->getQuery(true);
			$query->select('id')->from('#__{package}_lists')->where('db_table_name = '.$db->Quote($tableName));
			$db->setQuery($query);
			$id = $db->loadResult();
			$dispatcher = JDispatcher::getInstance();
			$elementModel = new plgFabrik_Element($dispatcher);

			if ($id) {
				//a fabrik table already exists - so we can copy the formatting of its elements
				$groupTableModel = JModel::getInstance('list', 'FabrikFEModel');

				$groupTableModel->setId($id);
				$table = $groupTableModel->getTable();
				//$this->formModel = null; //reset form so that it loads new table form
				$groups = $groupTableModel->getFormGroupElementData();
				$newElements = array();
				$ecount = 0;
				foreach ($groups as $groupModel) {
					// if we are saving a new table and the previously found tables group is a join
					// then don't add its elements to the table as they don't exist in the database table
					// we are linking to
					if ($groupModel->isJoin() && JRequest::getCmd('task') == 'save' && JRequest::getInt('id') == 0) {
						continue;
					}
					$elementModels =& $groupModel->getMyElements();
					foreach ($elementModels as $elementModel) {
						$ecount++;
						$element =& $elementModel->getElement();
						$copy = $elementModel->copyRow($element->id, '', $groupId);
						$newElements[$element->id] =  $copy->id;
					}

				}
				foreach ($newElements as $origId => $newId) {
					$plugin = $pluginManager->getElementPlugin($newId);
					$plugin->finalCopyCheck($newElements);
				}
				// hmm table with no elements - lets create them from the structure anyway
				if ($ecount == 0) {
					$this->makeElementsFromFields($groupId, $tableName);
				}
			} else {
				//no previously found fabrik list
				$this->makeElementsFromFields($groupId, $tableName);
			}
		}
	}

	/**
	 * take a table name and make elements for all of its fields
	 * @param int group id
	 * @param string $tableName
	 */

	function makeElementsFromFields($groupId, $tableName)
	{
		$fabrikDb =& $this->getFEModel()->getDb();
		$dispatcher = JDispatcher::getInstance();
		$elementModel = new plgFabrik_Element($dispatcher);
		$pluginManager = $this->getFormModel()->getPluginManager();
		$user = JFactory::getUser();
		$elementTypes = JRequest::getVar('elementtype', array(), 'default', 'array');
		$safeTableName = FabrikString::safeColName($tableName);
		$fields = $fabrikDb->getTableFields(array($safeTableName));
		$fields = $fields[$safeTableName];
		$createdate = JFactory::getDate()->toMySQL();

		$key = $this->getFEModel()->getPrimaryKeyAndExtra($tableName);

		// no existing fabrik table so we take a guess at the most
		//relavent element types to  create
		$elementLabels = JRequest::getVar('elementlabels', array(), 'default', 'array');
		foreach ($fields as $label => $type) {
			$element = FabTable::getInstance('Element', 'FabrikTable');
			if (array_key_exists($ordering, $elementTypes)) {
				//if importing from a CSV file then we have userselect field definitions
				$plugin = $elementTypes[$ordering];
			} else {
				//if the field is the primary key and it's an INT type set the plugin to be the fabrik internal id
				if ($key[0]['colname'] == $label && strtolower(substr($key[0]['type'], 0, 3)) === 'int') {
					$plugin = 'internalid';
				} else {
					//otherwise guestimate!
					switch ( $type )
					{
						case "int" :
						case "tinyint" :
						case "varchar" :
							$plugin = 'field';
							break;
						case "text" :
						case "tinytext" :
						case "mediumtext" :
						case "longtext" :
							$plugin = 'textarea';
							break;
						case "datetime" :
						case "date" :
						case "time" :
						case "timestamp" :
							$plugin = 'date';
							break;
						default :
							$plugin = 'field';
							break;
					}
				}
			}
			$element->plugin 						= $plugin;
			$element->hidden 						= $element->label == 'id' ? '1' : '0';
			$element->group_id 					= $groupId;
			$element->name				 			= $label;
			$element->created 					= $createdate;
			$element->created_by 				= $user->get('id');
			$element->created_by_alias 	= $user->get('username');
			$element->published 						= '1';
			$element->show_in_list_summary = '1';
			switch ($plugin) {
				case 'textarea':
					$element->width = '40';
					break;
				case 'date':
					$element->width = '10';
					break;
				default:
					$element->width = '30';
					break;
			}
			$element->height 	= '6';
			$element->ordering 	= $ordering;
			$element->params 	= $elementModel->getDefaultAttribs();
			$element->label 	= JArrayHelper::getValue($elementLabels, $ordering, str_replace("_", " ", $label));

			if (!$element->store()) {
				return JError::raiseError(500, $element->getError());
			}

			$elementModel =& $pluginManager->getPlugIn($element->plugin, 'element');
			$elementModel->setId($element->id);
			$elementModel->_element = $element;
			// hack for user element
			$details = array(
					'group_id' => $element->group_id
			);
			JRequest::setVar('details', $details);

			$elementModel->onSave();
			$ordering ++;
		}
	}

	/**
	 * when saving a table that links to a database for the first time we
	 * automatically create a form to allow the update/creation of that tables
	 * records
	 * @access private
	 * @param int form id to copy from. If = 0 then create a default form. If not 0 then copy the form id passed in
	 * @return object form model
	 */

	private function createLinkedForm($formid = 0)
	{
		$config	= JFactory::getConfig();
		$user	= JFactory::getUser();
		$this->getFormModel();
		// $$$ rob required otherwise the JTable is loaed with db_table_name as a property
		//which then generates an error - not sure why its loaded like that though?
		$this->formModel->getForm();
		if ($formid == 0) {
			jimport('joomla.utilities.date');
			$createdate = JFactory::getDate();
			$createdate = $createdate->toMySQL();

			$form = $this->getTable('Form');
			$item =& $this->getTable();
			$form->label = $item->label;
			$form->record_in_database = 1;

			$form->created 						= $createdate;
			$form->created_by 				= $user->get('id');
			$form->created_by_alias 	= $user->get('username');
			$form->error							= JText::_('COM_FABRIK_FORM_ERROR_MSG_TEXT');
			$form->submit_button_label 	= JText::_('SAVE');
			$form->published							= $item->published;
			$form->form_template			= 'default';
			$form->view_only_template	= 'default';

			if (!$form->store()) {
				return JError::raiseError(500, $form->getError());
			}
			$this->setState('list.form_id', $form->id);
			$this->formModel->setId($form->id);

		} else {
			$this->setState('list.form_id', $formid);
			$this->formModel->setId($formid);
			$this->formModel->copy();
		}
		$this->formModel->getForm();
		return $this->formModel;
	}

	/**
	 * create a group
	 * used when creating a fabrik table from an existing db table
	 *
	 * NEW also creates the formgroup
	 *
	 * @access private
	 * @param array group data
	 * @param bol is the group a join default false
	 * @return int group id
	 */

	private function createLinkedGroup($data, $isJoin = false)
	{
		$user = JFactory::getUser();
		$createdate = JFactory::getDate();
		$group = $this->getTable('Group');
		$group->bind($data);
		$group->created 					= $createdate->toMySQL();
		$group->created_by 				= $user->get('id');
		$group->created_by_alias 	= $user->get('username');
		$group->published					= 1;

		$opts = new stdClass();
		$opts->repeat_group_button = 0;
		$opts->repeat_group_show_first = 1;
		$opts->repeat_group_js_add = '';
		$opts->repeat_group_js_delete ='';
		$group->params = json_encode($opts);

		$group->is_join = ($isJoin == true) ? 1 : 0;

		$group->store();
		if (!$group->store()) {
			JError::raiseError(500, $group->getError());
		}
		//create form group
		$formid = $this->getState('list.form_id');
		$formGroup = $this->getTable('FormGroup');
		$formGroup->form_id = $formid;
		$formGroup->group_id = $group->id;
		$formGroup->ordering = 999999;

		if (!$formGroup->store()) {
			JError::raiseError(500, $formGroup->getError());
		}
		$formGroup->reorder(" form_id = '$formid'");
		return $group->id;
	}

	/**
	 * test if the main J user can create mySQL tables
	 * @return bol
	 */

	private function canCreateDbTable()
	{
		return true;
		//@todo run create table test once when you install fabrik instead
		// dont use method below but simply try to create a table and if you cant give error
		// if you can remove tmp created table
		/*$db 		=& FabrikWorker::getDbo();
		$conf =& JFactory::getConfig();
		$host 		= $conf->getValue('config.host');
		$user 		= $conf->getValue('config.user');
		$db->setQuery("SELECT Create_priv FROM mysql.user WHERE (Host = '$host' OR Host = '%') AND user = '$user'");
		$res = $db->loadResult();
		if ($res == 'N' || is_null($res)) {
		return false;
		} else {
		return true;
		}*/
	}

	/**
	 * Method to copy one or more records.
	 *
	 * @param	array	$pks	An array of record primary keys.
	 *
	 * @return	boolean	True if successful, false if an error occurs.
	 * @since	1.6
	 */

	public function copy()
	{
		$db =& FabrikWorker::getDbo();
		$user =& JFactory::getUser();
		$pks = JRequest::getVar('cid', array(), 'default', 'array');
		foreach ($pks as $i => $pk) {

			$item =& $this->getTable();
			$item->load($pk);
			$item->id = 0;
			$formModel =& $this->createLinkedForm($item->form_id);
			$item->form_id = $formModel->getTable()->id;

			$createdate = JFactory::getDate();
			$createdate = $createdate->toMySQL();
			$item->created = $createdate;
			$item->modified = $db->getNullDate();
			$item->modified_by = $user->get('id');
			$item->hits = 0;
			$item->checked_out = 0;
			$item->checked_out_time = $db->getNullDate();
			if (!$item->store()) {
				$this->setError($item->getError());
				return false;
			}
			$this->setState('list.id', $table->id);
			//test for seeing if joins correctly stored when coping new table
			$this->copyJoins($pk, $table->id, $formModel->groupidmap);
		}
		return true;
	}

	/**
	 * when copying a table we need to copy its joins as well
	 * note that the group and elements already exists - just the join needs to be saved
	 * @param int $fromid table id to copy from
	 * @param int table id to copy to
	 * @param array group id map saying which groups got copied to which new group id (key = old id, value = new id)
	 * @return null
	 */

	function copyJoins($fromid, $toid, $groupidmap)
	{
		$db = FabrikWorker::getDbo();
		$query = $db->getQuery(true);
		$query->select('*')->from('#__{package}_joins')->where('list_id = '.(int)$fromid);
		$db->setQuery($query);
		$joins = $db->loadObjectList();
		foreach ($joins as $join) {
			$size = 10;
			$els =& $this->getFEModel()->getElements();
			foreach ($els as $el) {
				if( $el->getElement()->name == $join->table_key) {
					$size = JString::stristr($el->getFieldDescription(), 'int') ? '' : '10';
				}
			}
			$this->addIndex($join->table_key, 'join', 'INDEX', $size);
			$joinTable =$this->getTable('Join');
			$joinTable->load($join->id);
			$joinTable->id = 0;
			$joinTable->group_id = $groupidmap[$joinTable->group_id];
			$joinTable->list_id 		= $toid;
			if (!$joinTable->store()) {
				return JError::raiseWarning(500, $join->getError());
			}
		}
	}

	/**
	 * @depreciated
	 *
	 * replaces the table column names with a safer name - ie removes white
	 * space and none alpha numeric characters
	 */

	private function makeSafeTableColumns()
	{
		//going to test allowing non safe names - as they should be quoted when accessed
		return;
/*
		if (!$this->getFEModel()->canAlterFields()) {
			return;
		}
		$form = $this->getFormModel();
		$groups = $form->getGroupsHiarachy();
		$db = $this->getFEModel()->getDb();
		$post = JRequest::get('post');
		$tableName = ($post['jform']['db_table_name'] != '') ? $post['jform']['db_table_name'] : $post['jform']['_database_name'];
		$origColNames = $this->getFEModel()->getDBFields($tableName);
		foreach ($origColNames as $origColName) {
			$colName = strtolower($origColName->Field);
			$type = $origColName->Type;
			$fbConfig =& JComponentHelper::getParams('com_fabrik');
			if ($fbConfig->get('use_wip', false)) {
				// $$$ was causing some problems for a site that was foolishly using - in columns names
				$newColName = preg_replace("/[^A-Za-z0-9-]/", "_", $colName);
			}
			else {
				$newColName = preg_replace("/[^A-Za-z0-9]/", "_", $colName);
			}


			if ($colName != $newColName) {
				// $$$rob - if we change an underlying column name, then we need to also alter the coresponding
				// element name itself, otherwise things go hinky!
				foreach ($groups as $groupModel) {
					$elementModels =& $groupModel->getMyElements();
					foreach ($elementModels as $elementModel) {
						$element =& $elementModel->getElement();
						if ($element->name == $colName) {
							$element->name = $newColName;
							$element->store();
						}

					}
				}
				$sql = "ALTER TABLE ".FabrikString::safeColName($tableName)." CHANGE ".FabrikString::safeColName($colName)." ".FabrikString::safeColName($newColName)." $type";
				$db->setQuery($sql);
				if (!$db->query()) {
					JError::raiseWarning(500, $db->getErrorMsg());
				}
			}
		}*/
	}

	/**
	 * adds a primary key to the database table
	 * @param string the column name to make into the primary key
	 * @param bol is the column an auto incrementing number
	 * @param string column type definition (eg varchar(255))
	 */

	function updatePrimaryKey($fieldName, $autoIncrement, $type = 'int(11)')
	{
		$feModel = $this->getFEModel();
		if (!$feModel->canAlterFields()) {
			return;
		}
		$fabrikDatabase = $feModel->getDb();
		$post = JRequest::get('post');
		$tableName = ($post['jform']['db_table_name'] != '') ? $post['jform']['db_table_name'] : $post['jform']['_database_name'];
		$tableName = preg_replace('#[^0-9a-zA-Z_]#', '_', $tableName);
		$aPriKey = $feModel->getPrimaryKeyAndExtra($tableName);


		if (!$aPriKey) { // no primary key set so we should set it
			$this->addKey($fieldName, $autoIncrement, $type);
		} else {
			if (count($aPriKey ) > 1){
				// $$$ rob multi field pk - ignore for now
				return;
			}
			$aPriKey = $aPriKey[0];
			$shortKey = FabrikString::shortColName($fieldName);
			//$shortKey = $feModel->_shortKey($fieldName, true); // added true for second arg so it strips quotes, as was never matching colname with quotes
			if ($fieldName !=  $aPriKey['colname'] && $shortKey != $aPriKey['colname']) {
				$this->dropKey($aPriKey); // primary key already exists so we should drop it
				$this->addKey($fieldName, $autoIncrement, $type);
			} else {
				//update the key
				// $$$ hugh - only update it if we need to
				$priInc = $aPriKey['extra'] == 'auto_increment' ? '1' : '0';
				if ($priInc != $autoIncrement || $type != $aPriKey['type']) {
					$this->updateKey($fieldName, $autoIncrement, $type);
				}
			}
		}
	}

	/**
	 * internal function: add a key to the table
	 * @param string primary key column name
	 * @param bol is the column auto incrementing
	 * @param string the primary keys column type (if autoincrement true then int(6) is always used as the type)
	 */

	private function addKey($fieldName, $autoIncrement, $type = "INT(6)")
	{
		$db = $this->getFEModel()->getDb();
		$type      = $autoIncrement != true ? $type : 'INT(6)';
		//$table     =& $this->getTable();
		//$table->load($this->getState('list.id'));
		$post = JRequest::get('post');
		$tableName = ($post['jform']['db_table_name'] != '') ? $post['jform']['db_table_name'] : $post['jform']['_database_name'];
		$tableName = preg_replace('#[^0-9a-zA-Z_]#', '_', $tableName);
		$tableName = FabrikString::safeColName($tableName);
		$fieldName = FabrikString::shortColName($fieldName);
		if ($fieldName === "") {
			return false;
		}
		$sql = "ALTER TABLE ".$tableName." ADD PRIMARY KEY ($fieldName)";
		/* add a primary key */

		$db->setQuery($sql);
		if (!$db->query()) {
			return JError::raiseWarning(500, $db->getErrorMsg());
		}
		if ($autoIncrement) {
			$sql = "ALTER TABLE ".$tableName." CHANGE $fieldName $fieldName $type NOT NULL AUTO_INCREMENT"; //add the autoinc
			$db->setQuery($sql);
			if (!$db->query()) {
				return JError::raiseError(500, 'add key: ' . $db->getErrorMsg());
			}
		}
	}

	/**
	 * internal function: drop the table's key
	 * @param array existing key data
	 * @return bol true if ke droped
	 */

	private function dropKey( $aPriKey )
	{
		$db = $this->getFEModel()->getDb();
		//$table =& $this->getTable();
		//$table->load($this->getState('list.id'));
		$post = JRequest::get('post');
		$tableName = FabrikString::safeColName($post['jform']['db_table_name']);
		$sql = "ALTER TABLE ".$tableName." CHANGE " . FabrikString::safeColName($aPriKey['colname']) . ' '. FabrikString::safeColName($aPriKey['colname']) . ' '  . $aPriKey['type'] . " NOT NULL";
		/* removes the autoinc */
		$db->setQuery($sql);
		if (!$db->query()) {
			JError::raiseWarning(500, $db->getErrorMsg()) ;
			return false;
		}
		$sql = "ALTER TABLE ".$tableName." DROP PRIMARY KEY";
		/* drops the primary key */
		$db->setQuery($sql);
		if (!$db->query()) {
			JError::raiseWarning(500, 'alter table: ' . $db->getErrorMsg()) ;
			return false;
		}
		return true;
	}

	/**
	 * internal function: update an exisitng key in the table
	 * @param string primary key column name
	 * @param bol is the column auto incrementing
	 * @param string the primary keys column type
	 */

	function updateKey($fieldName, $autoIncrement, $type = "INT(11)")
	{
		$post = JRequest::get('post');
		$tableName = FabrikString::safeColName($post['jform']['db_table_name']);
		$db = $this->getFEModel()->getDb();
		if (strstr($fieldName, '.')) {
			$fieldName = array_pop(explode(".", $fieldName));
		}
		$table =& $this->getTable();
		$table->load($this->getState('list.id'));
		$sql = "ALTER TABLE ".$tableName." CHANGE ".FabrikString::safeColName($fieldName).' '.FabrikString::safeColName($fieldName)." $type NOT NULL ";
		/* update primary key */
		if ($autoIncrement) {
			$sql .= " AUTO_INCREMENT";
		}
		$db->setQuery($sql);
		if (!$db->query()) {
			$this->setError('update key:'.$db->getErrorMsg());
		}
	}

	/**
	 * translation has been turned off for the table so delete the content
	 * element xml file
	 */

	private function removeJoomfishXML()
	{
		$file = JPATH_ADMINISTRATOR.DS.'components'.DS.'com_joomfish'.DS.'contentelements'.DS.'fabrik-' . $this->getTable()->db_table_name . '.xml';
		if (JFile::exists($file)) {
			JFile::delete($file);
		}
	}

	/**
	 * write out the Joomfish contentelement xml file for the tables elements
	 * @return bol true if written out ok
	 */

	private function makeJoomfishXML()
	{
		$config =& JFactory::getConfig();
		$db =& FabrikWorker::getDbo();
		$elements =& $this->getElements();

		//get all database join elements and check if we need to create xml files for them

		$table =& $this->getTable();
		$tableName = str_replace($config->getValue('dbprefix'), '',$table->db_table_name);
		$params =& $this->getParams();
		$titleElement = $params->get('joomfish-title');
		$str = '<?xml version="1.0" ?>
<joomfish type="contentelement">
  <name>Fabrik - ' . $table->label . '</name>
  <author>rob@pollen-8.co.uk</author>
  <version>1.0 for Fabrik 2.0</version>
  <description>Definition for Fabrik Table data - ' . $table->label . '</description>
  <reference type="content">
  	<table name="' . $tableName . '">';
		$titleset = false;
		foreach ($elements as $element) {
			if ($table->db_primary_key == FabrikString::safeColName($element->getFullName(false, false, false)))  {
				//primary key element
				$type = 'referenceid';
				$t = 0;
			} else {
				if (!$titleset && $titleElement == '') {
					$type ='titletext';
					$titleset = true;
				} else {
					if ($titleElement == $element->getFullName(false, false, false)) {
						$type ='titletext';
						$titleset = true;
					} else {
						$type = $element->getJoomfishTranslationType();
					}
				}
				$t = $element->getJoomfishTranslatable();
			}
			$opts = $element->getJoomfishOptions();
			$el = $element->getElement();
			$str .= "\n\t\t" . '<field type="'.$type.'" name="'.$el->name.'" translate="'.$t.'"';
			foreach ($opts as $k=>$v) {
				$str .= " $k=\"$v\"";
			}
			$str .='>' . $el->label . '</field>';
		}
		$str .='
  	</table>
  </reference>
</joomfish>';
		//file name HAS to be the same as the table name MINUS db extension
		return JFile::write(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_joomfish'.DS.'contentelements'.DS.$tableName.'.xml', $str);
	}

	/**
	 * Method to delete one or more records.
	 *
	 * @param	array	$pks	An array of record primary keys.
	 *
	 * @return	boolean	True if successful, false if an error occurs.
	 * @since	1.6
	 */
	public function delete(&$pks)
	{
		// Initialise variables.
		$dispatcher	= JDispatcher::getInstance();
		$user		= JFactory::getUser();
		$pks		= (array) $pks;
		$table		= $this->getTable();
		$app = JFactory::getApplication();
		// Include the content plugins for the on delete events.
		JPluginHelper::importPlugin('content');

		$post = JRequest::get('post');
		$deleteDepth = $post['jform']['recordsDeleteDepth'];
		$drop = $post['jform']['dropTablesFromDB'];

		$feModel = $this->getFEModel();
		$fabrikDatabase = $feModel->getDb();
		$db = FabrikWorker::getDbo();
		$dbconfigprefix = JApplication::getCfg("dbprefix");
		// Iterate the items to delete each one.
		foreach ($pks as $i => $pk) {

			if ($table->load($pk)) {
				if ($this->canDelete($table)) {
					$context = $this->option.'.'.$this->name;

					// Trigger the onContentBeforeDelete event.
					$result = $dispatcher->trigger($this->event_before_delete, array($context, $table));
					if (in_array(false, $result, true)) {
						$this->setError($table->getError());
						return false;
					}

					if (!$table->delete($pk)) {
						$this->setError($table->getError());
						return false;
					}

					// Trigger the onContentAfterDelete event.
					$dispatcher->trigger($this->event_after_delete, array($context, $table));

					// get the tables form
				} else {

					// Prune items that you can't change.
					unset($pks[$i]);
					JError::raiseWarning(403, JText::_('JLIB_APPLICATION_ERROR_EDIT_STATE_NOT_PERMITTED'));
				}

				switch ($deleteDepth) {
					case 0: //list only
					default:
						break;
					case 1: //list and form
						$form = $this->deleteAssociatedForm($table);
						break;
					case 2://list form and groups
						$form = $this->deleteAssociatedForm($table);
						$this->deleteAssociatedGroups($form, false);
						break;
					case 3://list form groups and elements
						$form = $this->deleteAssociatedForm($table);
						$this->deleteAssociatedGroups($form, true);
						break;
				}

				if ($drop) {
					if (strncasecmp($table->db_table_name, $dbconfigprefix, strlen($dbconfigprefix))==0) {
						$app->enqueueMessage(JText::sprintf('COM_FABRIK_TABLE_NOT_DROPPED_PREFIX', $table->db_table_name, $dbconfigprefix), 'notice');
					} else {
						$fabrikDatabase->setQuery("DROP " . $table->db_table_name);
						$fabrikDatabase->query();
						$app->enqueueMessage(JText::sprintf('COM_FABRIK_TABLE_DROPPED', $table->db_table_name));
					}
				} else {
					$app->enqueueMessage(JText::sprintf('COM_FABRIK_TABLE_NOT_DROPPED', $table->db_table_name), 'notice');
				}
			} else {
				$this->setError($table->getError());
				return false;
			}
		}
		return true;
	}

	private function deleteAssociatedForm(&$table)
	{
		$db = FabrikWorker::getDbo();
		$query = $db->getQuery(true);
		$form = $this->getTable('form');
		$form->load($table->form_id);
		$query->delete()->from('#__{package}_forms')->where('id = ' . (int)$form->id);
		$db->setQuery($query);
		$db->query();
		return $form;
	}

	private function deleteAssociatedGroups(&$form, $deleteElements = false)
	{
		$db = FabrikWorker::getDbo();
		$query = $db->getQuery(true);
		//get group ids
		$query->select('group_id')->from('#__{package}_formgroup')->where('form_id = ' .$form->id);
		$db->setQuery($query);
		$groupids = (array)$db->loadResultArray();

		//delete groups
		if (!empty($groupids)) {
			$query->clear();
			$query->delete()->from('#__{package}_groups')->where('id IN (' . implode(',', $groupids) . ')');
			$db->setQuery($query);
			$db->query();

			//delete form group records
			$query->clear();
			$query->delete()->from('#__{package}_formgroups')->where('group_id IN (' . implode(',', $groupids) . ')');
			$db->setQuery($query);

			if ($deleteElements) {
				//get the element ids
				$query->clear();
				$query->select('id')->from('#__{package}_elements')->where('group_id IN (' . implode(',', $groupids) . ')');
				$db->setQuery($query);
				$elementids = $db->loadResultArray();

				//delete the elements
				$query->clear();
				$query->delete()->from('#__{package}_elements')->where('group_id IN (' . implode(',', $groupids) . ')');
				$db->setQuery($query);
				$db->query();

				//delete js actions
				$query->clear();
				$query->delete()->from('#__{package}_jsactions')->where('element_id IN (' . implode(',', $elementids) . ')');
				$db->setQuery($query);
				$db->query();
			}
		}
		return $form;
	}

	public function dbTableFromXML($key, $name, $xml)
	{
		$row = $xml[0];
		$data = array();
		//	get which field types to use
		foreach ($row->children() as $child) {
			$value = sprintf("%s", $child);
			$type = $child->attributes()->type;
			if ($type == '') {
				$objtype = strtotime($value) == false ? "VARCHAR(255)" : "DATETIME";
				if (strstr($value, "\n")) {
					$objtype = 'TEXT';
				}
			} else {
				switch (strtolower($type)) {
					case 'integer':
						$objtype = 'INT';
						break;
					case 'datetime':
						$objtype = "DATETIME";
						break;
					case 'float':
						$objtype = "DECIMAL(10,2)";
						break;
					default:
						$objtype = "VARCHAR(255)";
						break;
				}
			}

			$data[$child->getName()] = $objtype;
		}
		if (empty($data)) {
			return false;
		}
		$db = $this->_db;
		$query = $db->getQuery();
		$query = "CREATE TABLE IF NOT EXISTS " . $db->nameQuote($name) . " (";
		foreach ($data as $fname => $objtype) {
			$query .= $db->nameQuote($fname) . " $objtype, \n";
		}
		$query .= " primary key ($key))";
		$query .= " ENGINE = MYISAM ";
		$db->setQuery($query);
		$db->query();

		//get a list of existinig ids
		$query = $db->getQuery(true);
		$query->select($key)->from($name);
		$db->setQuery($query);
		$existingids = $db->loadResultArray();
		//build the row object to insert/update
		foreach ($xml as $row) {
			$o = new stdClass();
			foreach ($row->children() as $child) {
				$k = $child->getName();
				$o->$k = sprintf("%s", $child);
			}
			//either update or add records
			if (in_array($o->$key, $existingids)) {
				$db->updateObject($name, $o, $key);
			} else {
				$db->insertObject($name, $o, $key);
			}
		}
	}

}
?>